import java_cup.runtime.*;

%%

%class scanner
%unicode
%cup
%line
%column


%{
  private Symbol symbol(int type) {
    return new Symbol(type, yyline, yycolumn);
  }
  private Symbol symbol(int type, Object value) {
    return new Symbol(type, yyline, yycolumn, value);
	
  }
%}

nl = \r|\n|\r\n
ws = [ \t]
string = (\"[^0-9]*\")
id = ([0-9]*[a-zA-Z\?\.\%&!\^\$£\+_\/\<\>\*\\\[\]@#\-]+[a-zA-Z0-9\?\.\%&!\^\$£\+_\/\<\>\*\\\[\]\-@#]*)
integer =  ([\+\-]*[1-9][0-9]*|0)
double = (([0-9]+\.[0-9]*) | ([0-9]*\.[0-9]+)) (e|E('+'|'-')?[0-9]+)?
not_evaluate = ('[a-zA-Z\-]+)

%%
"("     {return symbol(sym.RO);}
")"     {return symbol(sym.RC);}
"="     {return symbol(sym.EQ);}
"+"     {return symbol(sym.PLUS);}
"-"     {return symbol(sym.MINUS);}
"*"     {return symbol(sym.STAR);}
"/"     {return symbol(sym.DIV);}
"<"     {return symbol(sym.MIN);}
">"     {return symbol(sym.MAJ);}
"<="    {return symbol(sym.MIN_EQ);}
">="    {return symbol(sym.MAJ_EQ);}


//Array 
make-array {return symbol(sym.MAKEARRAY);}
:element-type {return symbol(sym.ELEMENTTYPE);}
aref {return symbol(sym.AREF);}


logand  {return symbol(sym.LOGAND);}
logior  {return symbol(sym.LOGIOR);}
logxor  {return symbol(sym.LOGXOR);}

//Output
print   {return symbol(sym.PRINT);}
write   {return symbol(sym.WRITE);}

//Statements
if      {return symbol(sym.IF);}
loop   {return symbol(sym.LOOP);}
for     {return symbol(sym.FOR);}
from    {return symbol(sym.FROM);}
to      {return symbol(sym.TO);}
do      {return symbol(sym.DO);}
then    {return symbol(sym.THEN);}
return    {return symbol(sym.RETURN);}

//Functions
defvar {return symbol(sym.DEFVAR);}
defun   {return symbol(sym.DEFUN);}
setf    {return symbol(sym.SETF);}
setq    {return symbol(sym.SETQ);}

{integer} {return symbol(sym.INT, new Integer(yytext())); }
{double}  {return symbol(sym.DOUBLE, new Double(yytext()));}
{not_evaluate} {return symbol(sym.NOTEVAL, new String(yytext()));}
{id}      {return symbol(sym.ID, new String(yytext()));}
{string}  {return symbol(sym.STRING, new String(yytext()));}

";".* {;}


{ws}|{nl}       {;}

. {System.out.print("SCANNER ERROR: "+yytext());}
